unit DragDropInfo;

interface

uses Windows, Classes;

type TDragDropInfo = Class(TObject)
 private
  FNumFiles: UINT;
  FInClientArea: Boolean;
  FDropPoint: TPoint;
  FFileList: TStringList;
 public
  constructor Create(ANumFiles: UINT);
  destructor Destroy; override;
  property NumFiles: UINT read FNumFiles;
  property InClientArea: Boolean read FInClientArea;
  property DropPoint: TPoint read FDropPoint;
  property Files: TStringList read FFileList;
end;

function GetDroppedFiles(hDrop: THandle): TDragDropInfo;
procedure AcceptDroppedFiles(Handle: HWND);
procedure UnAcceptDroppedFiles(Handle: HWND);

implementation

uses ShellAPI;

constructor TDragDropInfo.Create(ANumFiles: UINT);
begin
 inherited Create;
 FNumFiles := ANumFiles;
 FFileList := TStringList.Create;
end;

destructor TDragDropInfo.Destroy;
begin
 FFileList.Free;
 inherited Destroy;
end;

function GetDroppedFiles(hDrop: THandle): TDragDropInfo;
var
 DragDropInfo: TDragDropInfo;
 TotalNumberOfFiles, nFileLength: integer;
 pszFileName: PChar;
 i: integer;
begin
 TotalNumberOfFiles := DragQueryFile(hDrop, $FFFFFFFF, Nil, 0);
 DragDropInfo := TDragDropInfo.Create(TotalNumberOfFiles);
 DragDropInfo.FInClientArea := DragQueryPoint(hDrop, DragDropInfo.FDropPoint);
 for i := 0 to (TotalNumberOfFiles - 1) do begin
  nFileLength := DragQueryFile(hDrop, i, Nil, 0) + 1;
  GetMem(pszFileName, nFileLength);
  DragQueryFile(hDrop, i, pszFileName, nFileLength);
  DragDropInfo.FFileList.Add(pszFileName);
  FreeMem(pszFileName, nFileLength);
 end;
 DragFinish(hDrop);
 Result := DragDropInfo;
end;

procedure AcceptDroppedFiles(Handle: HWND);
begin
 DragAcceptFiles(Handle, True);
end;

procedure UnAcceptDroppedFiles(Handle: HWND);
begin
 DragAcceptFiles(Handle, False);
end;

end.