unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, CheckLst, Buttons, Spin, Menus, ComCtrls, ShellAPI, Registry;

type
  TSiliLauncherForm = class(TForm)
    PopupMenu1: TPopupMenu;
    AboutSiliLauncher1: TMenuItem;
    LauncherDelayTimer: TTimer;
    AlwaysOnTop1: TMenuItem;
    BottomPanel: TPanel;
    TopPanel: TPanel;
    RightPanel: TPanel;
    LeftPanel: TPanel;
    MiddlePanel: TPanel;
    CountedSoFarLabel: TLabel;
    DelaySpinEdit: TSpinEdit;
    AppNameEdit: TEdit;
    DeleteButton: TButton;
    StartCountingButton: TButton;
    LaunchNowButton: TButton;
    CloseOnCompletionCheckBox: TCheckBox;
    AutoStartCounterCheckBox: TCheckBox;
    DelayListBox: TListBox;
    ShortcutListBox: TListBox;
    AppNameCheckListBox: TCheckListBox;
    LeftSplitter: TSplitter;
    RightSplitter: TSplitter;
    AddItem1: TMenuItem;
    EditItem1: TMenuItem;
    Delete1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure WMDropFiles(hDrop: THandle; hWindow: HWND);
    procedure AppMessage(var Msg: TMsg; var Handled: Boolean);
    procedure AboutSiliLauncher1Click(Sender: TObject);
    procedure AppNameCheckListBoxClick(Sender: TObject);
    procedure AppNameEditChange(Sender: TObject);
    procedure DeleteButtonClick(Sender: TObject);
    procedure DelaySpinEditChange(Sender: TObject);
    procedure ShortcutListBoxClick(Sender: TObject);
    procedure DelayListBoxClick(Sender: TObject);
    procedure LauncherDelayTimerTimer(Sender: TObject);
    procedure StartCountingButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure AlwaysOnTop1Click(Sender: TObject);
    procedure LaunchNowButtonClick(Sender: TObject);
    procedure AppNameCheckListBoxDblClick(Sender: TObject);
    procedure AddItem1Click(Sender: TObject);
    procedure EditItem1Click(Sender: TObject);
  private
    { Private declarations }
    procedure Read_Registry;
    procedure Write_Registry;
  public
    { Public declarations }
    procedure UpdateItemIndices(NewIndex:integer);
  end;

const
  SECTION = 'SiliLauncher';

var
  SiliLauncherForm: TSiliLauncherForm;

implementation

uses AboutBox, DragAndDropInfo, EntryEdit;

var
  CurrentItem: integer;
  TimeCountedSoFar: integer;
  FIniFile: TRegIniFile;
  NumPrograms: integer;
  NumCheckedItems: integer;

{$R *.DFM}

procedure TSiliLauncherForm.AppMessage(var Msg: TMsg; var Handled: Boolean);
begin
 case Msg.Message of
  WM_DROPFILES:
   begin
    WMDropFiles(Msg.wParam, Msg.hWnd);
    Handled:=True;
   end;
  else
   Handled:=False;
 end;
end;

procedure TSiliLauncherForm.WMDropFiles(hDrop: THandle; hWindow: HWND);
var
 DragDropInfo: TDragDropInfo;
 i: integer;
begin
 DragDropInfo := DragAndDropInfo.GetDroppedFiles(hDrop);
 {if DragDropInfo.InClientArea then
  DoSomethingHere;}
 for i:= 0 to DragDropInfo.Numfiles - 1 do
  begin
   AppNameCheckListBox.Items.Append(DragDropInfo.Files[i]);
   ShortcutListBox.Items.Append(DragDropInfo.Files[i]);
   DelayListBox.Items.Append('10');
   AppNameCheckListBox.Checked[i]:=true;
   UpdateItemIndices(AppNameCheckListBox.Items.Count-1);
  end;
 DeleteButton.Enabled:=true;
 DragDropInfo.Free;
end;

procedure TSiliLauncherForm.Write_Registry;
var
 i: integer;
begin
  for i:=1 to NumPrograms do
   begin
    FIniFile.DeleteKey(SECTION, 'ProgramName'+IntToStr(i));
    FIniFile.DeleteKey(SECTION, 'ProgramPathName'+IntToStr(i));
    FIniFile.DeleteKey(SECTION, 'ProgramDelayTime'+IntToStr(i));
   end;

  NumPrograms:=AppNameCheckListBox.Items.Count;

  for i:=1 to NumPrograms do
   begin
    FIniFile.WriteString(SECTION, 'ProgramName'+IntToStr(i), AppNameCheckListBox.Items[i-1]);
    FIniFile.WriteString(SECTION, 'ProgramPathName'+IntToStr(i), ShortcutListBox.Items[i-1]);
    FIniFile.WriteString(SECTION, 'ProgramDelayTime'+IntToStr(i),   DelayListBox.Items[i-1]);
    FIniFile.WriteBool(  SECTION, 'ProgramEnabled'+IntToStr(i), AppNameCheckListBox.Checked[i-1]);
   end;

  FIniFile.WriteInteger(SECTION, 'NumPrograms', NumPrograms);

  FIniFile.WriteBool(SECTION, 'CloseOnCompletion', CloseOnCompletionCheckBox.Checked);
  FIniFile.WriteBool(SECTION, 'AutoStartCounter', AutoStartCounterCheckBox.Checked);
end;

procedure TSiliLauncherForm.Read_Registry;
var
 i: integer;
begin
  NumPrograms := FIniFile.ReadInteger(SECTION, 'NumPrograms', 0);
  CloseOnCompletionCheckBox.Checked:=FIniFile.ReadBool(SECTION, 'CloseOnCompletion', True);
  AutoStartCounterCheckBox.Checked:=FIniFile.ReadBool(SECTION, 'AutoStartCounter', False);

  if NumPrograms>0 then
   for i:=1 to NumPrograms do
    begin
     AppNameCheckListBox.Items.Append(FIniFile.ReadString(SECTION, 'ProgramName'+IntToStr(i), '[error]'));
     AppNameCheckListBox.Checked[AppNameCheckListBox.Items.Count-1]:=(FIniFile.ReadBool(SECTION, 'ProgramEnabled'+IntToStr(i), False));
     ShortcutListBox.Items.Append(FIniFile.ReadString(SECTION, 'ProgramPathName'+IntToStr(i), '[error]'));
     DelayListBox.Items.Append(FIniFile.ReadString(SECTION, 'ProgramDelayTime'+IntToStr(i), '9999'));
    end;

  UpdateItemIndices(CurrentItem);
end;

procedure TSiliLauncherForm.AboutSiliLauncher1Click(Sender: TObject);
begin
  AboutBoxForm.Show;
end;

procedure TSiliLauncherForm.AppNameCheckListBoxClick(Sender: TObject);
begin
  ShortcutListBox.ItemIndex := AppNameCheckListBox.ItemIndex;
  DelayListBox.ItemIndex    := AppNameCheckListBox.ItemIndex;

  if (AppNameCheckListBox.ItemIndex<>CurrentItem) then
   begin
    AppNameCheckListBox.Items[CurrentItem] := AppNameEdit.Text;
    DelayListBox.Items[CurrentItem]        := IntToStr(DelaySpinEdit.Value);
   end;
  UpdateItemIndices(AppNameCheckListBox.ItemIndex);
end;

procedure TSiliLauncherForm.ShortcutListBoxClick(Sender: TObject);
begin
  AppNameCheckListBox.ItemIndex := ShortcutListBox.ItemIndex;
  DelayListBox.ItemIndex        := ShortcutListBox.ItemIndex;

  if (AppNameCheckListBox.ItemIndex<>CurrentItem) then
   begin
    AppNameCheckListBox.Items[CurrentItem] := AppNameEdit.Text;
    DelayListBox.Items[CurrentItem]        := IntToStr(DelaySpinEdit.Value);
   end;
  UpdateItemIndices(AppNameCheckListBox.ItemIndex);
end;

procedure TSiliLauncherForm.DelayListBoxClick(Sender: TObject);
begin
  AppNameCheckListBox.ItemIndex := DelayListBox.ItemIndex;
  ShortcutListBox.ItemIndex     := DelayListBox.ItemIndex;

  if (AppNameCheckListBox.ItemIndex<>CurrentItem) then
   begin
    AppNameCheckListBox.Items[CurrentItem] := AppNameEdit.Text;
    DelayListBox.Items[CurrentItem]        := IntToStr(DelaySpinEdit.Value);
   end;
  UpdateItemIndices(AppNameCheckListBox.ItemIndex);
end;

procedure TSiliLauncherForm.FormCreate(Sender: TObject);
begin
 Application.OnMessage := AppMessage;
 DragAndDropInfo.AcceptDroppedFiles(Handle);
 UpdateItemIndices(0);
 TimeCountedSoFar:=0;
 FIniFile := TRegIniFile.Create('Software\SiliSoftware\SiliLauncher');
 Read_Registry;
 if AutoStartCounterCheckBox.Checked then
  StartCountingButtonClick(self);
end;

procedure TSiliLauncherForm.UpdateItemIndices(NewIndex:integer);
var
 i: integer;
begin
 CurrentItem                   := NewIndex;
 AppNameCheckListBox.ItemIndex := CurrentItem;
 ShortcutListBox.ItemIndex     := AppNameCheckListBox.ItemIndex;
 DelayListBox.ItemIndex        := AppNameCheckListBox.ItemIndex;
 if (AppNameCheckListBox.Items.Count>0) then
  begin
   AppNameEdit.Text              := AppNameCheckListBox.Items[NewIndex];
   DelaySpinEdit.Value           := StrToInt(DelayListBox.Items[NewIndex]);
   DelaySpinEdit.Enabled         := true;
   DeleteButton.Enabled          := true;
  end;

 NumCheckedItems:=0;
 for i:=0 to (AppNameCheckListBox.Items.Count-1) do
  if ((AppNameCheckListBox.Checked[i]) and
      (AppNameCheckListBox.ItemEnabled[i])) then
   NumCheckedItems:=NumCheckedItems+1;
end;

procedure TSiliLauncherForm.AppNameEditChange(Sender: TObject);
begin
 AppNameCheckListBox.Items[CurrentItem] := AppNameEdit.Text;
end;

procedure TSiliLauncherForm.DelaySpinEditChange(Sender: TObject);
begin
 DelayListBox.Items[CurrentItem] := IntToStr(DelaySpinEdit.Value);
end;

procedure TSiliLauncherForm.DeleteButtonClick(Sender: TObject);
begin
 AppNameEdit.Text:='';
 DelaySpinEdit.Value:=0;
 DelaySpinEdit.Enabled:=false;

 AppNameCheckListBox.Items.Delete(CurrentItem);
 ShortcutListBox.Items.Delete(CurrentItem);
 DelayListBox.Items.Delete(CurrentItem);
 if (CurrentItem>(AppNameCheckListBox.Items.Count-1)) then
  UpdateItemIndices(AppNameCheckListBox.Items.Count-1)
 else
  UpdateItemIndices(CurrentItem);

 if (AppNameCheckListBox.Items.Count=0) then
  begin
   UpdateItemIndices(0);
   DeleteButton.Enabled:=false;
  end
 else
  DeleteButton.Enabled:=true;
end;

procedure TSiliLauncherForm.LauncherDelayTimerTimer(Sender: TObject);
var
 i: integer;
begin
 TimeCountedSoFar:=TimeCountedSoFar+1;
 CountedSoFarLabel.Caption:=IntToStr(TimeCountedSoFar)+' seconds';
 Application.Title:='SiliLauncher - '+CountedSoFarLabel.Caption;

 if (AppNameCheckListBox.Items.Count>0) then
  for i:=0 to (AppNameCheckListBox.Items.Count-1) do
   begin
    if (TimeCountedSoFar>=StrToInt(DelayListBox.Items[i])) then
     if AppNameCheckListBox.Checked[i] then
      if AppNameCheckListBox.ItemEnabled[i] then
       begin
        ShellExecute(Application.Handle,
                     PChar('open'),
                     PChar(ShortcutListBox.Items[i]),
                     nil,
                     nil,
                     SW_SHOWNORMAL);
        AppNameCheckListBox.ItemEnabled[i]:=false;
        UpdateItemIndices(i);
       end;
   end;

 if (NumCheckedItems=0) then
  begin
   if CloseOnCompletionCheckBox.Checked then
    Close
   else
    StartCountingButtonClick(self);
  end;
end;

procedure TSiliLauncherForm.StartCountingButtonClick(Sender: TObject);
begin
 if LauncherDelayTimer.Enabled then
  begin
   StartCountingButton.Caption:='Start Counting';
   LauncherDelayTimer.Enabled:=false;
  end
 else
  begin
   StartCountingButton.Caption:='Stop Counting';
   LauncherDelayTimer.Enabled:=true;
  end;
end;

procedure TSiliLauncherForm.FormClose(Sender: TObject;  var Action: TCloseAction);
begin
  Write_Registry;
end;

procedure TSiliLauncherForm.AlwaysOnTop1Click(Sender: TObject);
begin
  AlwaysOnTop1.Checked:=not AlwaysOnTop1.Checked;

  if AlwaysOnTop1.Checked then
    SiliLauncherForm.FormStyle:=fsStayOnTop
  else
    SiliLauncherForm.FormStyle:=fsNormal;
end;

procedure TSiliLauncherForm.LaunchNowButtonClick(Sender: TObject);
begin
      ShellExecute(Application.Handle,
                   PChar('open'),
                   PChar(ShortcutListBox.Items[CurrentItem]),
                   nil,
                   nil,
                   SW_SHOWNORMAL);
end;

procedure TSiliLauncherForm.AppNameCheckListBoxDblClick(Sender: TObject);
begin
 AppNameCheckListBoxClick(self);
 EditEntryForm.AppTitleEdit.Text    := AppNameCheckListBox.Items[AppNameCheckListBox.ItemIndex];
 EditEntryForm.CommandLineEdit.Text :=     ShortcutListBox.Items[AppNameCheckListBox.ItemIndex];
 EditEntryForm.DelayEdit.Text       :=        DelayListBox.Items[AppNameCheckListBox.ItemIndex];
 EditEntryForm.ShowModal;
end;

procedure TSiliLauncherForm.AddItem1Click(Sender: TObject);
var
 NewIndex: integer;
begin
 NewIndex := AppNameCheckListBox.Items.Add('NewApp');
 NewIndex :=     ShortcutListBox.Items.Add('NewApp');
 NewIndex :=        DelayListBox.Items.Add('10');
 AppNameCheckListBox.Checked[NewIndex]:=true;
 UpdateItemIndices(AppNameCheckListBox.Items.Count-1);
 DeleteButton.Enabled:=true;

 AppNameCheckListBoxDblClick(self);
end;

procedure TSiliLauncherForm.EditItem1Click(Sender: TObject);
begin
 AppNameCheckListBoxDblClick(self);
end;

end.
